// HTTP RPC

// Copyright (C) 1999 by Makoto Nagata, MicroNova <mnagata@micronova.com>.  All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.
//
// Visit http://www.micronova.com for updates

package com.micronova.util;

import com.micronova.util.Packet;

import netscape.application.*;
import netscape.util.*;
import java.net.*;
import java.io.*;

/**

   HTTPPacket class provides a simple HTTP remote procedure call.
   It has no Codable implementation.

   * @version 1.0
   * @author Makoto Nagata, MicroNova

*/

public class HTTPPacket
{
    /**

       This opens a connection to a url, sends a codable, receives
       a codable, closes the connection, and returns the received
       codable.

    */

    public static Codable call(URL url, Codable codable) throws Exception
    {
        OutputStream out = null;
        InputStream in = null;

        try
        {
            URLConnection connection = url.openConnection();
    
            connection.setDoInput(true);
            connection.setDoOutput(true);
    
            // send argument

            out = connection.getOutputStream();

            Packet.write(out, codable);
    
            out.close();

            out = null;
    
            // receive result

            in = connection.getInputStream();

            codable = Packet.read(in);

            in.close();

            in = null;
            
            return codable;
        }
        catch (Exception e)
        {
            if (out != null)
            {
                out.close();
            }
            
            if (in != null)
            {
                in.close();
            }

            throw e;
        }
    }
}

