// Packet

// Copyright (C) 1999 by Makoto Nagata, MicroNova <mnagata@micronova.com>.  All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.
//
// Visit http://www.micronova.com for updates

package com.micronova.util;

import netscape.application.*;
import netscape.util.*;
import java.net.*;
import java.io.*;

/**

   Packet class provides static methods for transfering multiple
   Codable objects over a stream.  It has no Codable implementation.

   * @version 1.0
   * @author Makoto Nagata, MicroNova

*/

public class Packet
{
    /**

       Writes a Codable object to an OutputStream.  Objects are archived using binary mode.
       If the object is null, an empty record (of size 0) is written to the stream.

    */

    public static void write(OutputStream outputStream, Codable object) throws Exception
    {
        DataOutputStream oStream = new DataOutputStream(outputStream);

        ByteArrayOutputStream bStream = new ByteArrayOutputStream();

        if (object == null)
        {
            oStream.writeInt(0);
        }
        else
        {
            Archiver archiver = new Archiver(new Archive());
            archiver.archiveRootObject(object);

            archiver.archive().write(bStream);

            int size = bStream.size();

            oStream.writeInt(size);

            oStream.write(bStream.toByteArray(), 0, size);
        }

        byte[] endofline = {'\r', '\n'};

        oStream.write(endofline, 0, endofline.length);

        oStream.flush();
    }

    /**

       Reads a Codable from an InputStream.  Returns null if an
       empty record (of size 0) is read from the inputStream.

    */

    public static Codable read(InputStream inputStream) throws Exception
    {
        DataInputStream iStream = new DataInputStream(inputStream);

        Codable packet = null;

        int size = iStream.readInt();

        if (size != 0)
        {
            byte[] buffer = new byte[size];

            iStream.readFully(buffer, 0, buffer.length);

            ByteArrayInputStream bStream = new ByteArrayInputStream(buffer);

            Archive archive = new Archive();

            archive.read(bStream);

            int[] rootIds = archive.rootIdentifiers();

            if (rootIds != null)
            {
                Unarchiver unarchiver = new Unarchiver(archive);
            
                packet = (Codable)unarchiver.unarchiveIdentifier(rootIds[0]);
            }
        }

        byte[] endofline = new byte[2];
        iStream.readFully(endofline, 0, 2);

        return packet;
    }
}

